@@ -1,6 +1,6 @@
 # -*- mode: perl -*-
 #
-# $Id: LDAP.pm,v 1.2 1999/10/10 15:04:03 tai Exp $
+# $Id: LDAP.pm,v 1.3 2000/03/28 13:15:56 tai Exp $
 #
 
 package Tie::LDAP;
@@ -14,22 +14,22 @@ package Tie::LDAP;
  use Tie::LDAP;
 
  tie %LDAP, 'Tie::LDAP', {
-     host => $host, # LDAP hostname
-     user => $user, # DN used to access LDAP database
-     pass => $pass, # Password used with DN for access
+     host => $host, # LDAP hostname (defaults to 127.0.0.1)
+     port => $port, # Port number (defaults to 389)
+     user => $user, # Full DN used to access LDAP database
+     pass => $pass, # Password used with above DN
      base => $base, # Base DN used for each/keys/values operation
  };
 
 =head1 DESCRIPTION
 
-This library allows you to tie LDAP database to simple hash.
+This library allows you to tie LDAP database to Perl hash.
 Once tied, all hash operation will cause corresponding LDAP
-operation, hopefully as expected.
+operation, as you would (probably) expect.
 
-Fetching data from tied hash will return hash reference to
-named (by full DN) LDAP entry, which hold the attributes in
-lowercase as a key, and the references to ARRAY holding
-the real values as a value.
+Referencing tied hash will return hash reference to named
+LDAP entry that holds lowercased attribute as hash key, and
+reference to ARRAY containing data as hash value.
 
 Storing data is as easy as fetching: just push hash reference
 - with the same structure as fetched hash - back in.
@@ -38,9 +38,51 @@ Also, fetching/storing data into fetched hash reference will
 work as expected - it will manipulate corresponding field in
 fetched LDAP entry.
 
+=head1 EXAMPLE
+
+Here's a brief example of how you can use this module:
+
+  use Tie::LDAP;
+
+  ## connect
+  tie %LDAP, 'Tie::LDAP', { base => 'o=IMASY, c=JP' };
+
+  ## lookup entry for [dn: cn=tai, o=IMASY, c=JP]
+  $info = $LDAP{q{cn=tai, o=IMASY, c=JP}};
+
+  ## lookup each attributes
+  $user = $info->{username}->[0];
+  $mail = @{$info->{mailaddr}};
+
+  ## update each attributes
+  $info->{username} = ['newname'];
+  $info->{mailaddr} = ['tai@imasy.or.jp', 'tyamada@tk.elec.waseda.ac.jp'];
+
+  ## update entry
+  $LDAP{q{cn=tai, o=IMASY, c=JP}} = {
+    username => ['newname'],
+    mailaddr => ['tai@imasy.or.jp', 'tyamada@tk.elec.waseda.ac.jp'],
+  };
+
+  ## dump database (under base DN of [o=IMASY, c=JP]) in LDIF style
+  while (my($dn, $hash) = each %LDAP) {
+    print "dn: $dn\n";
+    while (my($name, $list) = each %{$hash}) {
+      foreach (@{$list}) {
+        print "$name: $_\n";
+      }
+    }
+    print "\n";
+  }
+
+  ## disconnect
+  untie %LDAP;
+
 =cut
 
 use strict;
+#use diagnostics;
+
 use Carp;
 use Net::LDAPapi;
 use Tie::LDAP::Entry;
@@ -48,12 +90,14 @@ use Tie::LDAP::Entry;
 use vars qw($DEBUG $VERSION);
 
 $DEBUG   = 0;
-$VERSION = '0.02';
+$VERSION = '0.05';
 
 sub TIEHASH {
     my $name = shift;
     my $opts = shift;
-    my $conn = new Net::LDAPapi($opts->{host} || 'localhost') || croak($@);
+    my $port = $opts->{port} || 389;
+    my $host = $opts->{host} || '127.0.0.1';
+    my $conn = new Net::LDAPapi($opts->{host}, $opts->{port}) || croak($@);
     my $mesg;
 
     print STDERR "[$name] TIEHASH\n" if $DEBUG;
@@ -71,7 +115,7 @@ sub FETCH {
     my $self = shift;
     my $path = shift;
     my $conn = $self->{conn};
-    my $mesg = $conn->search($path, LDAP_SCOPE_BASE, '(!(dn=))', [], 0);
+    my $mesg = $conn->search($path, LDAP_SCOPE_BASE, '(!( = ))', [], 0);
     my $data = {};
 
     print STDERR "[$self] FETCH\n" if $DEBUG;
@@ -185,17 +229,16 @@ sub DESTROY {
 
 =head1 BUGS
 
-Doing each/keys/values operation to tied hash works, but could
-be _very_ slow, depending on the size of the database.
+Doing each/keys/values operation to tied hash works (as shown in
+example), but could be _very_ slow, depending on the size of the
+database. This is because all operation is done synchronously.
 
-Also, though this is not a bug, substituting empty array to
-tied hash will cause whole database to be cleared out. I
-might disable this feature since almost nobody would want
-to see this happen anyway.
+Also, though this is not a bug, substituting empty array
+to tied hash will cause whole database to be cleared out.
 
 =head1 COPYRIGHT
 
-Copyright 1998-1999, T. Yamada <tai@imasy.or.jp>.
+Copyright 1998-2000, T. Yamada <tai@imasy.or.jp>.
 All rights reserved.
 
 This program is free software; you can redistribute it
@@ -203,7 +246,7 @@ and/or modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Net::LDAP>
+L<Net::LDAPapi>
 
 =cut
 
